@@ -304,23 +304,33 @@ def shape_enclosed_box(centre, shape):
 
     """
 
+    # The margin
+    margin = np.array(shape.get("margin", (0, 0, 0)))
+
     def cube_enclosed_box(cube):
         length = cube["length"]
-        return ((0, 0, 0), (length, length, length))
+        return (
+            np.array((0, 0, 0)) + margin,
+            np.array((length, length, length)) - margin,
+        )
 
     def cuboid_enclosed_box(cuboid):
         length_x = cuboid["length_x"]
         length_y = cuboid["length_y"]
         length_z = cuboid["length_z"]
-        return ((0, 0, 0), (length_x, length_y, length_z))
+        return (
+            np.array((0, 0, 0)) + margin,
+            np.array((length_x, length_y, length_z)) - margin,
+        )
 
     def cylinder_enclosed_box(cylinder):
         length = cylinder["length"]
         radius = np.mean(cylinder["radius"])
-        return (
-            (radius * (1 - 1 / sqrt(2)), 0, radius * (1 - 1 / sqrt(2))),
-            (radius * (1 + 1 / sqrt(2)), length, radius * (1 + 1 / sqrt(2))),
-        )
+        x0 = -(radius - margin[0]) / sqrt(2) + radius
+        x1 = (radius - margin[0]) / sqrt(2) + radius
+        z0 = -(radius - margin[2]) / sqrt(2) + radius
+        z1 = (radius - margin[2]) / sqrt(2) + radius
+        return ((x0, 0 + margin[1], z0), (x1, length - margin[1], z1))
 
     # The enclosed box
     x0, x1 = np.array(
@@ -334,11 +344,8 @@ def shape_enclosed_box(centre, shape):
     # The offset
     offset = centre - (x1 + x0) / 2.0
 
-    # The margin
-    margin = np.array(shape.get("margin", (0, 0, 0)))
-
     # Return the bounding box
-    return (x0 + offset + margin, x1 + offset - margin)
+    return (x0 + offset, x1 + offset)
 
 
 def is_shape_inside_box(box, centre, shape):
@@ -877,6 +884,8 @@ class SampleHDF5Adapter(object):
             Set the positions
 
             """
+            if "positions" in self.__handle:
+                del self.__handle["positions"]
             self.__handle.create_dataset("positions", data=positions, dtype="float32")
 
         @property
@@ -893,6 +902,8 @@ class SampleHDF5Adapter(object):
             Set the orientations
 
             """
+            if "orientations" in self.__handle:
+                del self.__handle["orientations"]
             self.__handle.create_dataset(
                 "orientations", data=orientations, dtype="float32"
             )
@@ -936,6 +947,13 @@ class SampleHDF5Adapter(object):
             """
             return self.keys()
 
+        def __contains__(self, key):
+            """
+            Check if molecule name exists in list
+
+            """
+            return key in self.__handle.keys()
+
         def keys(self):
             """
             Iterate through the molecule names
@@ -1341,9 +1359,19 @@ class Sample(object):
         # Add the molecule to it's own field
         if name is not None:
             name = name.replace("/", "_").replace("\\", "_").replace(".", "_")
-            self.__handle.sample.molecules[name].atoms = atoms.data
-            self.__handle.sample.molecules[name].positions = positions
-            self.__handle.sample.molecules[name].orientations = orientations
+            if name not in self.__handle.sample.molecules:
+                self.__handle.sample.molecules[name].atoms = atoms.data
+                self.__handle.sample.molecules[name].positions = positions
+                self.__handle.sample.molecules[name].orientations = orientations
+            else:
+                positions = np.concatenate(
+                    [self.__handle.sample.molecules[name].positions, positions]
+                )
+                orientations = np.concatenate(
+                    [self.__handle.sample.molecules[name].orientations, orientations]
+                )
+                self.__handle.sample.molecules[name].positions = positions
+                self.__handle.sample.molecules[name].orientations = orientations
 
     @property
     def containing_box(self):
