@@ -12,7 +12,6 @@
 import logging
 import numpy as np
 import time
-import warnings
 import parakeet.config
 import parakeet.dqe
 import parakeet.freeze
@@ -21,31 +20,22 @@ import parakeet.inelastic
 import parakeet.io
 import parakeet.sample
 import parakeet.simulate
+from parakeet.config import Device
 from parakeet.simulate.simulation import Simulation
+from parakeet.simulate.engine import SimulationEngine
 from parakeet.microscope import Microscope
 from parakeet.scan import Scan
 from functools import singledispatch
 from math import pi
-from collections.abc import Iterable
 from scipy.spatial.transform import Rotation as R
 
 
 __all__ = ["cbed"]
 
 
-Device = parakeet.config.Device
-ClusterMethod = parakeet.config.ClusterMethod
-Sample = parakeet.sample.Sample
-
 # Get the logger
 logger = logging.getLogger(__name__)
 
-# Try to input MULTEM
-try:
-    import multem
-except ImportError:
-    warnings.warn("Could not import MULTEM")
-
 
 class CBEDImageSimulator(object):
     """
@@ -58,92 +48,20 @@ class CBEDImageSimulator(object):
     """
 
     def __init__(
-        self, microscope=None, sample=None, scan=None, simulation=None, device="gpu"
+        self,
+        microscope=None,
+        sample=None,
+        scan=None,
+        simulation=None,
+        device="gpu",
+        gpu_id=None,
     ):
         self.microscope = microscope
         self.sample = sample
         self.scan = scan
         self.simulation = simulation
         self.device = device
-
-    def get_masker(
-        self,
-        index,
-        input_multislice,
-        pixel_size,
-        drift,
-        origin,
-        offset,
-        orientation,
-        shift,
-    ):
-        """
-        Get the masker object for the ice specification
-
-        """
-
-        # Create the masker
-        masker = multem.Masker(input_multislice.nx, input_multislice.ny, pixel_size)
-
-        # Get the sample centre
-        shape = self.sample.shape
-        centre = np.array(self.sample.centre)
-        drift = np.array(drift)
-        detector_origin = np.array([origin[0], origin[1], 0])
-        centre = centre + offset - detector_origin - shift
-
-        # Set the shape
-        if shape["type"] == "cube":
-            length = shape["cube"]["length"]
-            masker.set_cuboid(
-                (
-                    centre[0] - length / 2,
-                    centre[1] - length / 2,
-                    centre[2] - length / 2,
-                ),
-                (length, length, length),
-            )
-        elif shape["type"] == "cuboid":
-            length_x = shape["cuboid"]["length_x"]
-            length_y = shape["cuboid"]["length_y"]
-            length_z = shape["cuboid"]["length_z"]
-            masker.set_cuboid(
-                (
-                    centre[0] - length_x / 2,
-                    centre[1] - length_y / 2,
-                    centre[2] - length_z / 2,
-                ),
-                (length_x, length_y, length_z),
-            )
-        elif shape["type"] == "cylinder":
-            radius = shape["cylinder"]["radius"]
-            if not isinstance(radius, Iterable):
-                radius = [radius]
-            length = shape["cylinder"]["length"]
-            offset_x = shape["cylinder"].get("offset_x", None)
-            offset_z = shape["cylinder"].get("offset_z", None)
-            axis = shape["cylinder"].get("axis", (0, 1, 0))
-            if offset_x is None:
-                offset_x = [0] * len(radius)
-            if offset_z is None:
-                offset_z = [0] * len(radius)
-            masker.set_cylinder(
-                (centre[0], centre[1] - length / 2, centre[2]),
-                axis,
-                length,
-                list(radius),
-                list(offset_x),
-                list(offset_z),
-            )
-
-        # Rotate unless we have a single particle type simulation
-        if self.scan.is_uniform_angular_scan:
-            masker.set_rotation(centre, (0, 0, 0))
-        else:
-            masker.set_rotation(centre, orientation)
-
-        # Get the masker
-        return masker
+        self.gpu_id = gpu_id
 
     def __call__(self, index):
         """
@@ -190,35 +108,36 @@ class CBEDImageSimulator(object):
 
         # Create the multem system configuration
         system_conf = parakeet.simulate.simulation.create_system_configuration(
-            self.device
+            self.device,
+            self.gpu_id,
         )
 
         # The Z centre
         z_centre = self.sample.centre[2]
 
         # Create the multem input multislice object
-        input_multislice = (
-            parakeet.simulate.simulation.create_input_multislice_diffraction(
-                self.microscope,
-                self.simulation["slice_thickness"],
-                self.simulation["margin"] + self.simulation["padding"],
-                "CBED",
-                z_centre,
-            )
+        simulate = SimulationEngine(
+            self.device,
+            self.gpu_id,
+            self.microscope,
+            self.simulation["slice_thickness"],
+            self.simulation["margin"] + self.simulation["padding"],
+            "CBED",
+            z_centre,
         )
 
         # Set the specimen size
-        input_multislice.spec_lx = x_fov + offset * 2
-        input_multislice.spec_ly = y_fov + offset * 2
-        input_multislice.spec_lz = self.sample.containing_box[1][2]
+        simulate.input.spec_lx = x_fov + offset * 2
+        simulate.input.spec_ly = y_fov + offset * 2
+        simulate.input.spec_lz = self.sample.containing_box[1][2]
 
         # Set the beam tilt
-        input_multislice.theta += beam_tilt_theta
-        input_multislice.phi += beam_tilt_phi
+        simulate.input.theta += beam_tilt_theta
+        simulate.input.phi += beam_tilt_phi
 
         # Compute the B factor
         if self.simulation["radiation_damage_model"]:
-            input_multislice.static_B_factor = (
+            simulate.input.static_B_factor = (
                 8
                 * pi**2
                 * (
@@ -228,7 +147,7 @@ class CBEDImageSimulator(object):
                 )
             )
         else:
-            input_multislice.static_B_factor = 0
+            simulate.input.static_B_factor = 0
 
         # Set the atoms in the input after translating them for the offset
         atoms = self.sample.get_atoms()
@@ -259,7 +178,7 @@ class CBEDImageSimulator(object):
             atoms.data = atoms.data[select]
 
         # Translate for the detector
-        input_multislice.spec_atoms = atoms.translate(
+        simulate.input.spec_atoms = atoms.translate(
             (offset - origin[0], offset - origin[1], 0)
         ).to_multem()
         logger.info("   Got spec atoms")
@@ -280,9 +199,8 @@ class CBEDImageSimulator(object):
 
         if self.simulation["ice"] == True:
             # Get the masker
-            masker = self.get_masker(
+            masker = simulate.get_masker(
                 index,
-                input_multislice,
                 pixel_size,
                 drift,
                 origin,
@@ -292,22 +210,21 @@ class CBEDImageSimulator(object):
             )
 
             # Run the simulation
-            output_multislice = multem.simulate(system_conf, input_multislice, masker)
+            image = simulate.image(masker)
 
         else:
             # Set the incident wave
-            input_multislice.iw_x = [0]  # input_multislice.spec_lx/2
-            input_multislice.iw_y = [0]  # input_multislice.spec_ly/2
+            simulate.input.iw_x = [0]  # simulate.input.spec_lx/2
+            simulate.input.iw_y = [0]  # simulate.input.spec_ly/2
 
             # Run the simulation
             logger.info("Simulating")
-            output_multislice = multem.simulate(system_conf, input_multislice)
+            image = simulate.image(masker)
 
         # Get the ideal image data
         # Multem outputs data in column major format. In C++ and Python we
         # generally deal with data in row major format so we must do a
         # transpose here.
-        image = np.array(output_multislice.data[0].m2psi_tot).T
         x0 = padding
         y0 = padding
         x1 = image.shape[1] - padding
@@ -354,9 +271,8 @@ def simulation_factory(
     microscope: Microscope,
     sample: parakeet.sample.Sample,
     scan: Scan,
-    device: parakeet.config.Device = parakeet.config.Device.gpu,
     simulation: dict = None,
-    cluster: dict = None,
+    multiprocessing: dict = None,
 ) -> Simulation:
     """
     Create the simulation
@@ -365,9 +281,8 @@ def simulation_factory(
         microscope (object); The microscope object
         sample (object): The sample object
         scan (object): The scan object
-        device (str): The device to use
         simulation (object): The simulation parameters
-        cluster (object): The cluster parameters
+        multiprocessing (object): The multiprocessing parameters
 
     Returns:
         object: The simulation object
@@ -376,6 +291,13 @@ def simulation_factory(
     # Get the margin
     margin = 0 if simulation is None else simulation.get("margin", 0)
 
+    # Check multiprocessing settings
+    if multiprocessing is None:
+        multiprocessing = {"device": "gpu", "nproc": 1, "gpu_id": 0}
+    else:
+        assert multiprocessing["nproc"] in [None, 1]
+        assert len(multiprocessing["gpu_id"]) == 1
+
     # Create the simulation
     return Simulation(
         image_size=(
@@ -384,13 +306,13 @@ def simulation_factory(
         ),
         pixel_size=microscope.detector.pixel_size,
         scan=scan,
-        cluster=cluster,
         simulate_image=CBEDImageSimulator(
             microscope=microscope,
             sample=sample,
             scan=scan,
             simulation=simulation,
-            device=device,
+            device=multiprocessing["device"],
+            gpu_id=multiprocessing["gpu_id"][0],
         ),
     )
 
@@ -400,9 +322,9 @@ def cbed(
     config_file,
     sample_file: str,
     image_file: str,
-    device: parakeet.config.Device = parakeet.config.Device.gpu,
-    cluster_method: parakeet.config.ClusterMethod = None,
-    cluster_max_workers: int = 1,
+    device: Device = None,
+    nproc: int = None,
+    gpu_id: list = None,
 ):
     """
     Simulate the image from the sample
@@ -411,9 +333,9 @@ def cbed(
         config_file: The config filename
         sample_file: The sample filename
         image_file: The image filename
-        device: The device to run on (CPU or GPU)
-        cluster_method: The cluster method to use (default None)
-        cluster_max_workers: The maximum number of cluster jobs
+        device: The device to run on (cpu or gpu)
+        nproc: The number of processes
+        gpu_id: The list of gpu ids
 
     """
 
@@ -422,11 +344,11 @@ def cbed(
 
     # Set the command line args in a dict
     if device is not None:
-        config.device = device
-    if cluster_max_workers is not None:
-        config.cluster.max_workers = cluster_max_workers
-    if cluster_method is not None:
-        config.cluster.method = cluster_method
+        config.multiprocessing.device = device
+    if nproc is not None:
+        config.multiprocessing.nproc = nproc
+    if gpu_id is not None:
+        config.multiprocessing.gpu_id = gpu_id
 
     # Print some options
     parakeet.config.show(config)
@@ -470,9 +392,8 @@ def _cbed_Config(
         microscope,
         sample,
         scan,
-        device=config.device,
         simulation=config.simulation.dict(),
-        cluster=config.cluster.dict(),
+        multiprocessing=config.multiprocessing.dict(),
     )
 
     # Create the writer
