@@ -12,24 +12,21 @@
 import logging
 import mrcfile
 import numpy as np
-import warnings
 import parakeet.config
 import parakeet.dqe
 import parakeet.freeze
 import parakeet.futures
 import parakeet.inelastic
 import parakeet.sample
+from parakeet.config import Device
 from parakeet.microscope import Microscope
 from parakeet.scan import Scan
 from parakeet.simulate.simulation import Simulation
+from parakeet.simulate.engine import SimulationEngine
 from functools import singledispatch
 from math import pi, floor
 from scipy.spatial.transform import Rotation as R
 
-Sample = parakeet.sample.Sample
-Device = parakeet.config.Device
-ClusterMethod = parakeet.config.ClusterMethod
-
 
 __all__ = ["potential"]
 
@@ -38,13 +35,6 @@ __all__ = ["potential"]
 logger = logging.getLogger(__name__)
 
 
-# Try to input MULTEM
-try:
-    import multem
-except ImportError:
-    warnings.warn("Could not import MULTEM")
-
-
 class ProjectedPotentialSimulator(object):
     """
     A class to do the actual simulation
@@ -63,6 +53,7 @@ class ProjectedPotentialSimulator(object):
         scan=None,
         simulation=None,
         device="gpu",
+        gpu_id=None,
     ):
         self.potential_prefix = potential_prefix
         self.microscope = microscope
@@ -70,6 +61,7 @@ class ProjectedPotentialSimulator(object):
         self.scan = scan
         self.simulation = simulation
         self.device = device
+        self.gpu_id = gpu_id
 
     def __call__(self, index):
         """
@@ -109,16 +101,13 @@ class ProjectedPotentialSimulator(object):
         x0 = (-offset, -offset)
         x1 = (x_fov + offset, y_fov + offset)
 
-        # Create the multem system configuration
-        system_conf = parakeet.simulate.simulation.create_system_configuration(
-            self.device
-        )
-
         # The Z centre
         z_centre = self.sample.centre[2]
 
         # Create the multem input multislice object
-        input_multislice = parakeet.simulate.simulation.create_input_multislice(
+        simulate = SimulationEngine(
+            self.device,
+            self.gpu_id,
             self.microscope,
             self.simulation["slice_thickness"],
             self.simulation["margin"],
@@ -127,9 +116,9 @@ class ProjectedPotentialSimulator(object):
         )
 
         # Set the specimen size
-        input_multislice.spec_lx = x_fov + offset * 2
-        input_multislice.spec_ly = y_fov + offset * 2
-        input_multislice.spec_lz = self.sample.containing_box[1][2]
+        simulate.input.spec_lx = x_fov + offset * 2
+        simulate.input.spec_ly = y_fov + offset * 2
+        simulate.input.spec_lz = self.sample.containing_box[1][2]
 
         # Set the atoms in the input after translating them for the offset
         atoms = self.sample.get_atoms_in_fov(x0, x1)
@@ -150,7 +139,7 @@ class ProjectedPotentialSimulator(object):
             atoms.data["z"] = coords[:, 2]
 
         origin = (0, 0)
-        input_multislice.spec_atoms = atoms.translate(
+        simulate.input.spec_atoms = atoms.translate(
             (offset - origin[0], offset - origin[1], 0)
         ).to_multem()
         logger.info("   Got spec atoms")
@@ -168,19 +157,8 @@ class ProjectedPotentialSimulator(object):
         )
         potential.voxel_size = tuple((pixel_size, pixel_size, slice_thickness))
 
-        def callback(z0, z1, V):
-            V = np.array(V)
-            zc = (z0 + z1) / 2.0
-            index = int(floor((zc - volume_z0) / slice_thickness))
-            print(
-                "Calculating potential for slice: %.2f -> %.2f (index: %d)"
-                % (z0, z1, index)
-            )
-            if index < potential.data.shape[0]:
-                potential.data[index, :, :] = V[margin:-margin, margin:-margin].T
-
-        # Run the simulation
-        multem.compute_projected_potential(system_conf, input_multislice, callback)
+        # Simulate the potential
+        simulate.potential(potential, volume_z0)
 
         # Compute the image scaled with Poisson noise
         return (index, None, None)
@@ -189,11 +167,10 @@ class ProjectedPotentialSimulator(object):
 def simulation_factory(
     potential_prefix: str,
     microscope: Microscope,
-    sample: Sample,
+    sample: parakeet.sample.Sample,
     scan: Scan,
-    device: Device = Device.gpu,
     simulation: dict = None,
-    cluster: dict = None,
+    multiprocessing: dict = None,
 ) -> Simulation:
     """
     Create the simulation
@@ -203,9 +180,8 @@ def simulation_factory(
         microscope: The microscope object
         sample: The sample object
         scan: The scan object
-        device: The device to use
         simulation: The simulation parameters
-        cluster: The cluster parameters
+        multiprocessing (object): The multiprocessing parameters
 
     Returns:
         object: The simulation object
@@ -214,6 +190,13 @@ def simulation_factory(
     # Get the margin
     margin = 0 if simulation is None else simulation.get("margin", 0)
 
+    # Check multiprocessing settings
+    if multiprocessing is None:
+        multiprocessing = {"device": "gpu", "nproc": 1, "gpu_id": 0}
+    else:
+        assert multiprocessing["nproc"] in [None, 1]
+        assert len(multiprocessing["gpu_id"]) == 1
+
     # Create the simulation
     return Simulation(
         image_size=(
@@ -222,14 +205,14 @@ def simulation_factory(
         ),
         pixel_size=microscope.detector.pixel_size,
         scan=scan,
-        cluster=cluster,
         simulate_image=ProjectedPotentialSimulator(
             potential_prefix=potential_prefix,
             microscope=microscope,
             sample=sample,
             scan=scan,
             simulation=simulation,
-            device=device,
+            device=multiprocessing["device"],
+            gpu_id=multiprocessing["gpu_id"][0],
         ),
     )
 
@@ -239,9 +222,9 @@ def potential(
     config_file,
     sample_file: str,
     potential_prefix: str,
-    device: Device = Device.gpu,
-    cluster_method: ClusterMethod = None,
-    cluster_max_workers: int = 1,
+    device: Device = None,
+    nproc: int = None,
+    gpu_id: list = None,
 ):
     """
     Simulate the projected potential from the sample
@@ -250,9 +233,9 @@ def potential(
         config_file: The input config filename
         sample_file: The input sample filename
         potential_prefix: The input potential filename
-        device: The device to run on (CPU or GPU)
-        cluster_method: The cluster method to use (default None)
-        cluster_max_workers: The maximum number of cluster jobs
+        device: The device to run on (cpu or gpu)
+        nproc: The number of processes
+        gpu_id: The list of gpu ids
 
     """
 
@@ -261,11 +244,11 @@ def potential(
 
     # Set the command line args in a dict
     if device is not None:
-        config.device = device
-    if cluster_max_workers is not None:
-        config.cluster.max_workers = cluster_max_workers
-    if cluster_method is not None:
-        config.cluster.method = cluster_method
+        config.multiprocessing.device = device
+    if nproc is not None:
+        config.multiprocessing.nproc = nproc
+    if gpu_id is not None:
+        config.multiprocessing.gpu_id = gpu_id
 
     # Print some options
     parakeet.config.show(config)
@@ -310,9 +293,8 @@ def _potential_Config(
         microscope=microscope,
         sample=sample,
         scan=scan,
-        device=config.device,
         simulation=config.simulation.dict(),
-        cluster=config.cluster.dict(),
+        multiprocessing=config.multiprocessing.dict(),
     )
 
     # Run the simulation
